const std = @import("std");

const _maxRead = 4096;

pub const BatteryInfo = struct {
	Manufacturer: []u8,
	Model: []u8,
	Technology: []u8,
        Status: []u8,
	CellSeriesCount: u32 = 0,
	Capacity: u32 = 0,
	CapacityDesign: u32 = 0,
	Vnow: f32 = 0,
	Vmin: f32 = 0,
	ChargeNow: u32 = 0,
	ChargeFull: u32 = 0,
	ChargeFullPercentageDesign: u32 = 0,
	ChargeFullDesign: u32 = 0,
	_allocator: std.mem.Allocator,

	pub fn deinit(self: *const BatteryInfo) void {
		self._allocator.free(self.Manufacturer);
		self._allocator.free(self.Model);
		self._allocator.free(self.Technology);
                self._allocator.free(self.Status);
	}
};

fn _readInfo(allocator: std.mem.Allocator, dir: std.fs.Dir, path: []const u8) ![]u8 {
	const start = try dir.readFileAlloc(allocator, path, _maxRead);
        defer allocator.free(start);
        const trimmed = std.mem.trim(u8, start, "\r\n\t ");
        return allocator.dupe(u8, trimmed);
}

fn _readInfoU32(allocator: std.mem.Allocator, dir: std.fs.Dir, path: []const u8) !u32 {
	const info = try _readInfo(allocator, dir, path);
	defer allocator.free(info);
	return std.fmt.parseInt(u32, info, 10);
}

pub fn GetBatteryInfoAlloc(allocator: std.mem.Allocator, dir: std.fs.Dir) !BatteryInfo {
	const manufacturer = try _readInfo(allocator, dir, "manufacturer");
	const model = try _readInfo(allocator, dir, "model_name");
	const technology = try _readInfo(allocator, dir, "technology");
        const status = try _readInfo(allocator, dir, "status");

	var info = BatteryInfo{
		.Manufacturer = manufacturer,
		.Model = model,
		.Technology = technology,
                .Status = status,
		._allocator = allocator,
	};

	info.Capacity = try _readInfoU32(allocator, dir, "capacity");
	info.ChargeNow = try _readInfoU32(allocator, dir, "energy_now") / 1000;
	info.ChargeFull = try _readInfoU32(allocator, dir, "energy_full") / 1000;
	info.ChargeFullDesign = try _readInfoU32(allocator, dir, "energy_full_design") / 1000;

	if(info.ChargeFullDesign != 0) {
		info.ChargeFullPercentageDesign = 100 * info.ChargeFull / info.ChargeFullDesign;
		info.CapacityDesign = 100 * info.ChargeNow / info.ChargeFullDesign;
	}

	const Vnow = try _readInfoU32(allocator, dir, "voltage_now") / 1000;
	const Vmin = try _readInfoU32(allocator, dir, "voltage_min_design") / 1000;

	info.Vnow = @as(f32, @floatFromInt(Vnow)) / 1000;
	info.Vmin = @as(f32, @floatFromInt(Vmin)) / 1000;

	info.CellSeriesCount = @intFromFloat(@ceil(info.Vmin / 3.7));

	return info;
}
